// MIS3502 - Project 3
// Created by: Jeremy Shafer
// Spring 2023

// NOTES ************************************************
// You should raise the default lambda timeout 
// from 3 seconds to 3 minutes

// declarations *****************************************
const qs = require('qs'); //for parsing URL encoded data
const axios = require('axios'); // for calling another API
const base64 = require('js-base64'); // encode / decode
const mysql = require('sync-mysql');  //for talking to a database


// Put your database connection information here.
const dboptions = {
  'user' : '',
  'password' : '',
  'database' : '',
  'host' : 'dataanalytics.temple.edu'
};

const connection = new mysql(dboptions);

const features = [
	"Issue a GET against ./auth and provide a username and password.  The API will respond with a JSON object containing all the user information except the password.",
	"Issue a POST against ./users to create a new user record. You must provide the following keys: username, password, email. The API will respond with the userid of the newly created record.",
	"Issue a GET against ./scancheck to check an event code. You must provide the following keys: eventcode, userid. The eventcode is a UUID. It is a string 36 characters in length. If the event code is good, the API will return a JSON object with the date and time of the newly created history record. The JSON object contains the date and time expressed as text, and as epoch time.",
	"UNDER CONSTRUCTION: Issue a GET against ./history to retrieve a JSON object containing record of all the events attended by the user. You must provide the following key: userid. The events in the repsonse will be ordered by thier creation date/time with the most resent attendance records being listed first.",
	"This API belongs to YOUR NAME HERE"
	];


// supporting functions *********************************
let authenticate = (res,query) => {
	
	// error trapping
	if (isEmpty(query)){
		return formatres(res,"You must provide a username and password.",400);
	}

    let username = query.username;
    let password = query.password;

    if (username == undefined || username.trim() == ""){
            return formatres(res,"You must provide a username",400);
    }

    if (password == undefined || password.trim() == "" || password.length < 8 ){
    	return formatres(res,"You must provide a username",400);
    }
	
	//do some work and return the result

    let txtSQL = "select userid, username, email, sms, usercreatedate, lastlogin from users where username = ? and password = ?";
    try {
	    var results = connection.query(txtSQL,[username,password]);
    }
    catch(e){
    	return formatres(res,'Unexpected error.  (authenticate)',500);
    }

    if (results.length == 0){
        return formatres(res,"Login Failed", 400);
    }

	return formatres(res,results,200);
}

let makeUser = (res,body) => {

	//error trapping
    let username = body.username;
    let password = body.password;
    let email = body.email;

    if (username == undefined || username.trim() == ""){
        return formatres(res,"You must provide a username",400);
    }

    if (password == undefined || password.trim() == "" || password.length < 8 ){
        return formatres(res,"You must provide a password",400);

    }

    let pos0 = password.indexOf(' ');
    
    if (pos0 != -1 ){
       return formatres(res,"The password may not contain blank spaces. ",400);
    }

    if (email == undefined || email.trim() == ""){
       return formatres(res,"You must provide an email",400);
    }

    let pos1 = email.indexOf('@');
    let pos2 = email.indexOf('.');

    if (pos1 == -1 || pos1 == 0 || pos1 == email.length){
        return formatres(res,"You must provide a valid email",400);
    }

    if (pos2 == -1 || pos2 == 0 || pos2 == email.length){
        return formatres(res,"You must provide a valid email",400);
    }

	// do some work and return the result

    let txtSql = "select count(*) as qty from users where username = ?";
    var results = connection.query(txtSql,[username]);
    if ( results[0]['qty'] > 0 ){
        return formatres(res,'Username already exists',400);
    }

    let txtSql2 = "insert into users (username,password,email) values (?,?,?)";

    try {
      var results2 = connection.query(txtSql2,[username,password,email]);
    }
    catch(e){
        console.log(e);
        return formatres(res,'Unexpected error. (makeUser)',500);
    }
    
    return formatres(res,results2.insertId,200);
}

let verifyEventCode = (res,query) => {
	// error trapping
    let userid = query.userid;
    let eventcode = query.eventcode;

    if (userid == undefined || userid == "" || isNaN(userid)  ){
        return formatres(res,"You must provide a userid.",400);
    }

    if (eventcode == undefined || eventcode == "" || eventcode.length != 36 || eventcode.indexOf("-") == -1  ){
        return formatres(res,"You must provide an event code.",400);
    }
	
	// do some work and return the result
    let txtSQL = "select * from events where eventcode = ? ";
    let txtSQL2 = "insert into history (eventid, userid) values (?,?)";
    let txtSQL3 = 'select DATE_FORMAT(scandate, "%M %d %Y %r") as scandate_text , UNIX_TIMESTAMP(scandate) as scandate_epoch from history where historyid = ?';
    try {
        var results = connection.query(txtSQL,[eventcode]);
        if (results.length > 0){
            let eventid = results[0]['eventid'];
            var results2 = connection.query(txtSQL2,[eventid,userid]);
            var results3 = connection.query(txtSQL3,[results2.insertId]);
        }
    }
    catch(e){
            return formatres(res,'Unexpected error.  (verifyEventCode)',500);
    }

    if (results.length == 0){
            return formatres(res,"This event does not exist: " + eventcode, 400);
    }

    return formatres(res,results3[0],200);
}

// do not delete this handy little supporting function
let formatres = (res, output, statusCode) => {
	res.statusCode = statusCode;
	res.body = JSON.stringify(output);
	return res;	
}

// do not delete this handy little supportng function
function isEmpty(obj) {
    return Object.keys(obj).length === 0;
}

// My Routing Function **********************************

let myRoutingFunction = (res,method,path,query,body) => {

	// conditional statements go here.
	// look at the path and method and return the output from the 
	// correct supporting function.

	// Simple GET request with no features specified results
	// in a list of features / instructions
	if (method == "GET" && path == ""){
		return formatres(res, features, 200);
	}
	
	if (method == "GET" && path == "auth"){
		return authenticate(res,query);
	}
	
	if (method == "POST" && path == "users"){
		return makeUser(res,body);
	}
	
	if (method == "GET" && path == "scancheck"){
		return verifyEventCode(res,query);
	}
	
	if (method == "GET" && path == "history"){
		//return getHistory(res,query);
		return formatres(res, "The history feature is under construction.", 200);
	}
	
	return(res);
}


// event handler ****************************************

// Students should not have to change the code here.
// Students should be able to read and understand the code here.

exports.handler = async (request) => {

	// identify the method (it will be a string)
	let method = request["httpMethod"];
	
	// identify the path (it will also be a string)
	let fullpath = request["path"];
	
	// we clean the full path up a little bit
	if (fullpath == undefined || fullpath == null){ fullpath = ""};
	let pathitems = fullpath.split("/");
	let path = pathitems[2];
	if (path == undefined || path == null){ path = ""};
	
	// identify the querystring ( we will convert it to 
	//   a JSON object named query)
	let query = request["queryStringParameters"];
	if (query == undefined || query == null){ query={} };
	
	// identify the body (we will convert it to 
	//   a JSON object named body)
	let body = qs.parse(request["body"]);
	if (body == undefined || body == null){ body={} };

	// Create the default response object that will include 
	// the status code, the headers needed by CORS, and
	// the string to be returned formatted as a JSON data structure.
    let res = {
        'statusCode': 400,
        'headers': {
            'Access-Control-Allow-Origin': '*',
            'Access-Control-Allow-Credentials': true  
        },
        'body': JSON.stringify("Feature not found."),
    };

	// run all the parameters through my routing function
	// and return the result
    return myRoutingFunction(res,method,path,query,body);
    
    //this is an example of Shafer testing one specific function:
    //body={"username":"adam","password":"Password123","email":"adam@email.com"}
    //return makeUser(res,body)
};